// -*- C++ -*-
/*!
 * @file  Urg.cpp * @brief hokuyo scip2 component * $Date$ 
 *
 * $Id$ 
 */
#include "Urg.h"
#include "scip2.h"
// Module specification
// <rtc-template block="module_spec">
static const char* urg_spec[] =
  {
    "implementation_id", "Urg",
    "type_name",         "Utm30lx",
    "description",       "hokuyo scip2 component",
    "version",           "1.0",
    "vendor",            "SGIJ",
    "category",          "Generic",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.str_port", "/dev/ttyACM0",
    ""
  };
// </rtc-template>

Urg::Urg(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_ScanOut("Scan", m_Scan),
    m_LRFInfoPort("LRFInfo")

    // </rtc-template>
{/*
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  // Set OutPort buffer
  registerOutPort("Scan", m_ScanOut);

  // Set service provider to Ports
  m_LRFInfoPort.registerProvider("LRF", "LRFInfo", m_LRF);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_LRFInfoPort);

  // </rtc-template>
  m_Scan.data.length(SCIP2_TOPURG_SAMPLES);*/
}

Urg::~Urg()
{
}


RTC::ReturnCode_t Urg::onInitialize()
{
    // </rtc-template>
//{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers

  // Set OutPort buffer
  addOutPort("Scan", m_ScanOut);

  // Set service provider to Ports
  m_LRFInfoPort.registerProvider("LRF", "LRFInfo", m_LRF);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_LRFInfoPort);

  // </rtc-template>
  m_Scan.data.length(SCIP2_TOPURG_SAMPLES);
//}

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("str_port", m_str_port, "/dev/ttyACM0");
  
  dev = NULL;
  memset(scan, 0, sizeof(int)*SCIP2_TOPURG_SAMPLES);

  // </rtc-template>
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Urg::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t Urg::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t Urg::onActivated(RTC::UniqueId ec_id)
{
  dev = Scip2_Start((char*)m_str_port.c_str());

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Urg::onDeactivated(RTC::UniqueId ec_id)
{
  Scip2_End(dev);
  dev = NULL;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Urg::onExecute(RTC::UniqueId ec_id)
{
  Scip2_GD(dev,0,SCIP2_TOPURG_SAMPLES-1,1, scan);

  for (CORBA::ULong i = 0; i < SCIP2_TOPURG_SAMPLES; i++) {
    m_Scan.data[i] = scan[i];

//   printf("%d ", scan[i]);
  }
//  printf("%ld \n",m_Scan.data[30]);
  m_ScanOut.write();

  usleep(10000);
  
  return RTC::RTC_OK;
}


extern "C"
{
 
  void UrgInit(RTC::Manager* manager)
  {
    RTC::Properties profile(urg_spec);
    manager->registerFactory(profile,
                             RTC::Create<Urg>,
                             RTC::Delete<Urg>);
  }
  
};



